local VERSION = tonumber(("$Revision$"):match("%d+"))

if uab.revision < VERSION then
	uab.version = "r" .. VERSION
	uab.revision = VERSION
	uab.date = ("$Date$"):match("%d%d%d%d%-%d%d%-%d%d")
end

--[[
Identifies the action information.  This includes the visual behavior of the 
associated action button (hidden, low alpha, blinking, etc.) and the action to 
execute when the button is clicked.  This class sets the action attributes on 
the ActionButton, and tells the ActionButton how to draw itself based on the 
visual effect settings and current unit state.  This class also controls 
whether the ActionButton is in use or not.  If not in use, the ActionButton is 
told to hide itself.

Each Action has one or more ActionButton instances associated with it. 
Each ActionButton instance is associated with a single Action instance.
here is one ActionButton for each action of the ActionSet associated with each 
UnitGroupHeader.
--]]
uabAction = uab:NewModule("Action")
uabAction.core = uab

function uabAction:OnInitialize()
    uabAction.super.OnInitialize(self)
end

function uabAction:CreateAction(number, data, ishostile)
	local actiondata
	local action

	if uab.classes[data.classname] then
		actiondata = uab.classes[data.classname]:new(data.data)
		action = uab.classes.Action:new(number, actiondata, data.useplayerunit, data.effect, ishostile)
	end

	return action
end

function uabAction.ItemUsable(button, usable)
    if usable then
        getglobal(button:Frame():GetName().."Icon"):SetVertexColor(1.0, 1.0, 1.0, 1.0)
    else
        getglobal(button:Frame():GetName().."Icon"):SetVertexColor(0.25, 0.25, 0.25, 1.0)
    end
end

function uabAction.AnyChosenEffect(settings)
    if settings then
		if settings.highlight or settings.lowalpha or
			settings.noalpha or settings.blink or
			settings.notusable or settings.showcooldown then
            return true
        end
    end
end
        
uab.classes.Action = uab.mixins.aceOO.Class()

function uab.classes.Action.prototype:init(number, data, useplayerunit, effects, ishostile)
	uab.classes.Action.super.prototype.init(self)

    self.ishostile = ishostile	
    self.enabled = true
	self.number = number
	self.buttons = {}
    self.attributes = {}
	self:SetAll(data, useplayerunit, effects)
end

function uab.classes.Action.prototype:Clear()
    self:SetAll(nil, nil, nil)    
    
	for button, _ in pairs(self.buttons) do
		if self.ishostile then
            button:DisableHostileAction()
        else
            button:DisableAction()
        end
	end
end

function uab.classes.Action.prototype:SetAll(data, useplayerunit, effects)
    self.useplayerunit = useplayerunit
	self.effects = effects
    self:SetData(data)
end

function uab.classes.Action.prototype:SetData(data)
	self:ClearButtonAttributes()
    self:ClearAttributes()
	
    self.data = data
    
    if self.data then    
        local attribs = self.data:GetAttributes()
        
        if not self.ishostile then
            for name, val in pairs(attribs) do
    		    self.attributes[name] = val
    	    end        
        else
            self.attributes["harmbutton"] = "hostile"
            
            --[[
            We have to make the attributes specific to hostile units only.
            --]]
    
            for name, val in pairs(attribs) do
    		    self.attributes[name.."-hostile"] = val
    	    end        
        end
    end
    
	self:SetButtonAttributes()
end

function uab.classes.Action.prototype:GetData()
	return self.data
end

function uab.classes.Action.prototype:SetEffects(effects)
	self.effects = effects
end

function uab.classes.Action.prototype:ShowButton(button)
    self:SetButtonIcon(button)
    self:SetButtonCount(button)
    
    self:UpdateVisualEffects(button)
end

function uab.classes.Action.prototype:AddButton(button)
    self.buttons[button] = true
	if self.data then
        button:SetAttributes(self.attributes)
    end
    self:SetButtonIcon(button)
    self:SetButtonCount(button)
end

function uab.classes.Action.prototype:RemoveButton(button)
	self.buttons[button] = nil
	if self.data then
        button:ClearAttributes(self.attributes)
    end
end

function uab.classes.Action.prototype:ShowTooltip(frame)
	if self.data and uab.db.profile.tooltips then
        self.data:ShowTooltip(frame)
    end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.Action.prototype:ClearAttributes()
    for name, val in pairs(self.attributes) do
        self.attributes[name] = nil
    end
end

function uab.classes.Action.prototype:ClearButtonAttributes()
	if self.data then
        local attribs = self.attributes
		for button, _ in pairs(self.buttons) do
            button:ClearAttributes(attribs)
		end
	end
end

function uab.classes.Action.prototype:SetButtonAttributes()
	if self.data then
        local attribs = self.attributes
		for button, _ in pairs(self.buttons) do
			button:SetAttributes(attribs)
		end
	end
end

function uab.classes.Action.prototype:SetButtonIcon(button)
    if self.data then
        local icon = self.data:GetIcon()
        
        if icon then
            local iconFrame = getglobal(button:Frame():GetName().."Icon")
            if iconFrame then
                local curIcon = iconFrame:GetTexture()
                if curIcon ~= icon then
                    iconFrame:SetTexture(icon)
                end
            end
        end
    end
end

function uab.classes.Action.prototype:SetButtonCount(button)
    btncount = getglobal(button:Frame():GetName().."Count")

    if self.data and self.data:GetCount() then
        btncount:Show()
        btncount:SetText(self.data:GetCount())
    else
        btncount:Hide()
    end
end

function uab.classes.Action.prototype:ClearEffects(button)
    for name, _ in pairs(button.visualeffect) do
        button.visualeffect[name] = nil
    end
end

function uab.classes.Action.prototype:AnyEffects(button)
    local effect = button.visualeffect
    
    for n, v in pairs(effect) do
        return true
    end
end

local defaultStr = "Default"
local oldDefaultStr = "default"

function uab.classes.Action.prototype:ApplyEffect(button)
    if self.sample then
        button:BorderColor(1.0, 1.0, 1.0, 0.0)
        button:SetAlpha(1.0)
        button:EndFlash()
    else
        if not self:AnyEffects(button) and self.effects.Default then
            self:CheckPriority(button, defaultStr)
        end

        local effect = button.visualeffect
        local usable

        usable = self:CheckItemIsUsable(button)

        if usable then
            usable = not effect.notusable
        end
        
        uabAction.ItemUsable(button, usable)
        
	    if effect.highlight then
	    	local color = effect.highlight.color

	        button:BorderColor(color.r, color.g, color.b, color.a)
	    else
	        button:BorderColor(1.0, 1.0, 1.0, 0.0)
	    end

        if effect.blink then
            button:StartFlash()
	    else
	        button:EndFlash()
        end

	    if effect.noalpha then
	        button:SetAlpha(0.0)
	    elseif effect.lowalpha then
	        button:SetAlpha(uab.db.profile.alpha)
	    else
            button:SetAlpha(1.0)
	    end
    end
end

function uab.classes.Action.prototype:CheckPriority(button, setting)
    local tocheck = self.effects[setting]
    
    if tocheck then
        local effect = button.visualeffect
    	local options = uabUnitStatus:GetUnitStatusConfig(setting)
        local newPriority = options.priority
        local highlight = options.color	
    
        if tocheck.highlight then
            if not effect.highlight or newPriority > effect.highlight.priority then
                effect.highlight = { color = highlight }
                effect.highlight.priority = newPriority
            end
        end
        
        if tocheck.lowalpha then
            effect.lowalpha = true
        end
        
        if tocheck.noalpha then
            effect.noalpha = true
        end
        
        if tocheck.blink then
            effect.blink = true
        end
        
        if tocheck.notusable then
            effect.notusable = true
        end
    end
end

function uab.classes.Action.prototype:CheckItemIsUsable(button)
	if self.data and uab.db.profile.checkusable then
        return self.data:IsUsable()
    else
        return true
    end
end

function uab.classes.Action.prototype:GetEffectInfo(button, unit, setting)
    if uabUnitStatus:IsStatusGained(setting, unit, self.data) then
        self:CheckPriority(button, setting)
    end
end

function uab.classes.Action.prototype:IsDefaultEffect(effect)
    return effect == oldDefaultStr or effect == defaultStr
end

function uab.classes.Action.prototype:GetCooldownOverride(start, duration, new_start, new_duration)
    if not start or start <= 0 or not duration or duration <= 0 then
        -- no original start or duration, use new if set
        if new_start and new_start > 0 and new_duration and new_duration > 0 then
            return new_start, new_duration
        end
    else
        -- we have an original start and duration.  Check if we have new values
        if new_start and new_start > 0 and new_duration and new_duration > 0 then
            -- we have both original and new.  Check to see which ends the latest and use that one
            if start + duration < new_start + new_duration then
                return new_start, new_duration
            end
        end
    end
    
    return start, duration
end

function uab.classes.Action.prototype:GetCooldown(button)
    local start, duration, enable = self:GetData():GetCooldown()

	if not self.sample then
        local unit = uab.GetFrameUnit(button:Frame())

        if self.useplayerunit then
            unit = "player"
        end
        
	    if unit and self.effects then
			for cureffect, settings in pairs(self.effects) do
                --[[
				No need to check for an effect on the unit if having the effect
				makes no change to any of the button attributes.
				--]]
			    if not self:IsDefaultEffect(cureffect) and uabAction.AnyChosenEffect(settings) then
                    local tocheck = self.effects[cureffect]
                    if tocheck and tocheck.showcooldown then
                        local hasbuff, tmp_enabled, tmp_start, tmp_duration = uabUnitStatus:IsStatusGained(cureffect, unit, self.data)
                        if hasbuff and tmp_enabled and tmp_enabled > 0 and tmp_start and tmp_start > 0 and tmp_duration and tmp_duration > 0 then
                            enable = 1
                            start, duration = self:GetCooldownOverride(start, duration, tmp_start, tmp_duration)
                        end
                    end
			    end
			end
		end
	end

    enable = enable and enable > 0 and start and start > 0 and duration and duration > 0 and 1

    return start, duration, enable
end

function uab.classes.Action.prototype:UpdateVisualEffects(button)
    self:ClearEffects(button)
    
	if not self.sample then
        local unit = uab.GetFrameUnit(button:Frame())

        if self.useplayerunit then
            unit = "player"
        end
        
	    if unit and self.effects then
			for cureffect, settings in pairs(self.effects) do
                --[[
				No need to check for an effect on the unit if having the effect
				makes no change to any of the button attributes.
				--]]
			    if not self:IsDefaultEffect(cureffect) and uabAction.AnyChosenEffect(settings) then
			        self:GetEffectInfo(button, unit, cureffect)
			    end
			end
		end
	end

    self:ApplyEffect(button)
end

